require 'oembed'
require 'cgi'


# Process the content of documents before rendering them to find URLs in
# a block.
Jekyll::Hooks.register :site, :pre_render do |site|
  # Cache results
  cache ||= Jekyll::Cache.new('Jekyll::OEmbed::Urls')

  # Only modify documents to be written
  site.docs_to_write.each do |doc|
    # Skip text paragraphs
    next unless %r{\n\nhttps?://} =~ doc.content

    # Split texts by markdown blocks
    doc.content = doc.content.split("\n\n").map do |p|
      # Only process lines with URLs
      if %r{\Ahttps?://} =~ p
        # Remove empty characters
        p.strip!

        # @see {https://developer.mozilla.org/en-US/docs/Web/HTML/Element/iframe#attr-sandbox}
        same_origin = p.start_with? site.config['url']

        Jekyll.logger.debug "Finding OEmbed content for #{p}"
        # Cache the results
        cache.getset(p) do
          Jekyll.logger.debug "=> Not cached, obtaining..."

          result = OEmbed::Providers.get(p)

          # Return a sandboxed iframe with the size of the HTML.  We
          # only allow scripts to run inside the iframe and nothing
          # else.
          <<~IFRAME
            <iframe
              referrerpolicy="no-referrer"
              sandbox="allow-scripts #{same_origin ? '' : 'allow-same-origin'}"
              style="min-width:#{result.width}px;min-height:#{result.height || 0}px"
              srcdoc="#{CGI.escape_html result.html}"
              ></iframe>
          IFRAME

          result.html
        rescue OEmbed::NotFound => e
          # If the URL doesn't support OEmbed just return an external
          # link.
          #
          # TODO: Fetch information with OGP and render a template.
          Jekyll.logger.warn "#{p} is not oembeddable or URL can't be fetched, showing as URL"

          "<a href=\"#{p}\" target=\"_blank\">#{p}</a>"
        end
      else
        # Otherwise return the original block
        p
      end
    # Rebuild the content
    end.join("\n\n")
  end
end
